module PHP
  module InheritAttributes
    # Merge the content of a databag into an attribute
    # @param databag [String] the databag::item value
    # @return attribute
    def merge_databag_into(attribute, databag)
      merge_attribute(attribute, load_databag(databag))
    end

    # Merge the content of a databag into self
    # @param databag [String] the databag::item value
    # @return nil
    def merge_databag(databag)
      load_databag(databag).each do |key, value|
        next if %w(data_bag).include?(key)
        merge_attribute(key, value) if respond_to?(key)
      end
    end

    # Load databag item
    # @param databag [String] the databag::item value
    # @return [Hash]
    def load_databag(databag, required: true)
      begin
        data_bag_item(*databag.split('::')).to_hash
      rescue StandardError
        raise if required
        Hash.new
      end
    end

    # Merge the content of a hash into self
    # @param hash [Hash]
    # @return nil
    def merge_hash(hash)
      hash.each do |key, value|
        merge_attribute(key, value) if respond_to?(key)
      end if hash
    end

    # Merge a hash into a resource attribute, thus extending it's contents
    # @param attribute [String] the name of the attribute
    # @param hash [Hash,NilClass] the data hash to extend with
    # @return [Mash]
    def merge_attribute(attribute, hash)
      data = deep_merge(send("#{attribute}"), hash)
      data = Mash.new(data) if data.is_a?(Hash)
      instance_variable_set("@#{attribute}", data)
    end

    # Merge two hashes together, creating a resulting Mash object
    #
    # *Note*: This deep_merge method requires that the hashes be provided 
    # in the same sequence as it would with Ruby's `.merge` method. 
    # Specifically, items with the highest precedence are to the right of the
    # method call. This is the opposite of how Chef::Mixin::DeepMerge works.
    #
    # @param source [Hash,NilClass]
    # @param data [Hash,NilClass]
    # @returns [Mash]
    # @since 1.0.1
    def deep_merge(source, data)
      Chef::Mixin::DeepMerge.deep_merge(data, source)
    end
  end
end
